﻿'use strict';

var callSummary = (function () {
    var CallStateEnum = {
        RINGING: 0,
        ANSWERED: 1,
        HANGUP: 2,
        properties: {
            0: { name: "Zvonění" },
            1: { name: "Hovor" },
            2: { name: "Zavěšeno" }
        }
    };

    var CallUserStateEnum = {
        AGENT_LOGIN: 0,
        AGENT_LOGOUT: 1,
        AGENT_PAUSED: 2,
        AGENT_UNPAUSED: 3,
        properties: {
            0: { name: "Přihlášení" },
            1: { name: "Odhlášení" },
            2: { name: "Pauza" },
            3: { name: "Konec pauzy" }
        }
    };

    var correctUrl = function (str) {
        if (!str)
            return str;
        if (str.indexOf("://") === -1)
            str = "//" + str;
        return str;
    };

    return {
        correctUrl: correctUrl,
        callUserStateEnum: CallUserStateEnum,
        callStateEnum: CallStateEnum,
        NO_PAUSE_ID: "w5cf0z90mqf173"
    };
})();

// view model
function CallSummaryViewModel(selectedCallItemUniqueId) {
    var self = this;
    var CALL_ITEMS_LIMIT = 100;

    self.isLoading = ko.observable(true);
    self.selectedCallItemUniqueId = ko.observable();
    self.callItems = ko.observableArray([]);
    self.callItemsOverallSum = ko.observable(0);
    self.processedCallItems = ko.observable();

    self.activeNotIgnoredCallItems = ko.pureComputed(function () {
        return $.grep(self.callItems(), function (callItem) {
            return callItem.state() !== callSummary.callStateEnum.HANGUP && !callItem.ignored();
        });
    }, this);

    self.lastCallItemIsHanguped = ko.pureComputed(function () {
        var callItemsLength = self.callItems().length;
        if (callItemsLength > 0)
            return self.callItems()[callItemsLength - 1].state() === callSummary.callStateEnum.HANGUP;

        return false;
    }, this);

    self.selectCallItem = function (uniqueId) {
        self.selectedCallItemUniqueId(uniqueId);
        var selectedCallItem = self.selectedCallItem();
        if (typeof callDashboard !== 'undefined' && selectedCallItem)
        {
            OnAjaxBegin();
            $.connection.callHub.server.getCallSession(selectedCallItem.uniqueId).done(function (callSession) {
                selectedCallItem.dispose();
                self.callItems.replace(selectedCallItem, new CallItem(callSession));
                OnAjaxComplete();
            }).fail(function (error) {
                console.warn(error);
                OnAjaxComplete();
            });
        }
    };

    self.selectedCallItem = ko.pureComputed(function () {
        if (self.selectedCallItemUniqueId())
        {
            return getCallItem(self.selectedCallItemUniqueId());
        }

        return null;
    }, this);

    function getCallItem(uniqueId) {
        for (let i = 0; i < self.callItems().length; i++) {
            let callItem = self.callItems()[i];
            if (callItem.uniqueId === uniqueId) {
                return callItem;
            }
        }

        return null;
    }

    self.makeCall = function (to, relatedTicketId, relatedUserProviderKey) {
        $.connection.callHub.server.makeCall(to, relatedTicketId, relatedUserProviderKey).fail(function (error) {
            console.warn(error);
        });
    };

    self.processAllCallItems = function () {
        return new Promise(function (resolve, reject) {
            $.connection.callHub.server.processAllCallSessions().done(function () {
                selectDefaultCallItem();
                resolve();
            }).fail(function (error) {
                console.warn(error);
                reject(error);
            });
        });
    };

    self.queueRelations = ko.observable();
    self.pauses = ko.observable();
    self.selectedPause = ko.observable();
    self.selectPause = function () {
        var selectedPause = self.selectedPause();
        if (selectedPause !== callSummary.NO_PAUSE_ID) {
            $.connection.callHub.server.sumPause(selectedPause).fail(function (error) {
                console.warn(error);
            });
        }
        else {
            $.connection.callHub.server.sumUnpause().fail(function (error) {
                console.warn(error);
            });
        }
    };

    self.isAnyPaused = ko.pureComputed(function () {
        if (self.queueRelations()) {
            for (var i = 0; i < self.queueRelations().length; i++) {
                if (self.queueRelations()[i].isPaused() === true) {
                    return true;
                }
            }
        }

        return false;
    }, this);

    self.isAnyLogged= ko.pureComputed(function () {
        if (self.queueRelations()) {
            for (var i = 0; i < self.queueRelations().length; i++) {
                if (self.queueRelations()[i].isLogged() === true) {
                    return true;
                }
            }
        }

        return false;
    }, this);

    function setCallItem(callSession)
    {
        var callItem = getCallItem(callSession.UniqueId);
        if (!callItem)
        {
            if (callSession.State !== callSummary.callStateEnum.HANGUP)
            {
                if (self.callItems().length >= CALL_ITEMS_LIMIT) {
                    self.callItems.pop();
                }

                self.callItems.unshift(new CallItem(callSession));
                self.callItemsOverallSum(self.callItemsOverallSum() + 1);
                selectDefaultCallItem();
            }
        }
        else
        {
            if (callSession.State === callSummary.callStateEnum.HANGUP && callItem.state() === callSummary.callStateEnum.RINGING) {
                self.callItems.remove(callItem);
                self.callItemsOverallSum(self.callItemsOverallSum() - 1);
                selectDefaultCallItem();
            }
            else
                callItem.state(callSession.State);
        }
            
    }

    function setQueueRelation(callEventName, queue, pauseName) {
        if (self.queueRelations()) {
            for (var i = 0; i < self.queueRelations().length; i++) {
                let queueRelation = self.queueRelations()[i];
                if (queueRelation.name === queue) {
                    switch (callEventName) {
                        case callSummary.callUserStateEnum.AGENT_LOGIN:
                            queueRelation.setNewIsLogged(true);
                            break;
                        case callSummary.callUserStateEnum.AGENT_LOGOUT:
                            queueRelation.setNewIsLogged(false);
                            break;
                        case callSummary.callUserStateEnum.AGENT_PAUSED:
                            queueRelation.isPaused(true);
                            self.selectedPause(pauseName);
                            break;
                        case callSummary.callUserStateEnum.AGENT_UNPAUSED:
                            queueRelation.isPaused(false);
                            self.selectedPause(callSummary.NO_PAUSE_ID);
                            break;
                    }
                    return;
                }
            }
        }
    }

    function callItemsContainsUniqueId(uniqueId) {
        var callItem = getCallItem(uniqueId);
        return callItem ? true : false;
    }

    function selectDefaultCallItem() {
        if (self.selectedCallItemUniqueId() && callItemsContainsUniqueId(self.selectedCallItemUniqueId())) {
            return;
        }

        if (selectedCallItemUniqueId && callItemsContainsUniqueId(selectedCallItemUniqueId)) {
            self.selectCallItem(selectedCallItemUniqueId);
            return;
        }

        if (self.callItems().length > 0)
            self.selectCallItem(self.callItems()[0].uniqueId);
        else
            self.selectCallItem(null);
    }

    self.ignoreCurrentCallSessions = function () {
        $.connection.callHub.server.ignoreCurrentCallSessions().fail(function (error) {
            console.warn(error);
        });
    };

    function CategoryData(categoryData, parentCallItem, subscriptions) {
        var cd = this;
        cd.uniqueId = parentCallItem.uniqueId;
        cd.id = categoryData.Id;
        cd.val = ko.observable(categoryData.Val).extend({ required: categoryData.Required });
        
        var firstSub = true;
        subscriptions.push(cd.val.subscribe(function (newVal) {
            if (!firstSub) {
                $.connection.callHub.server.updateCallSessionCategory(cd.uniqueId, cd.id, newVal).fail(function (error) {
                    console.warn(error);
                });
            }
            else
                firstSub = false;
        }));
    }

    function CallFieldData(callFieldData, parentCallItem, subscriptions) {
        var cfd = this;
        cfd.uniqueId = parentCallItem.uniqueId;
        cfd.id = callFieldData.Id;
        cfd.val = ko.observable(callFieldData.Val).extend({ rateLimit: 1000, required: callFieldData.Required });
        subscriptions.push(cfd.val.subscribe(function (newVal) {
            $.connection.callHub.server.updateCallSessionField(cfd.uniqueId, cfd.id, newVal).fail(function (error) {
                console.warn(error);
            }); 
        }));
    }

    function CallItem(callSession) {
        var ci = this;
        var subscriptions = [];
        ci.uniqueId = callSession.UniqueId;
        ci.state = ko.observable(callSession.State);
        ci.from = callSession.From;
        ci.to = callSession.To;
        ci.line = callSession.Line;
        ci.email = callSession.Email;
        ci.login = callSession.Login;
        ci.fileName = callSession.FileName;
        ci.way = callSession.Way,
        ci.ringingDate = callSession.RingingDateUtc ? moment.utc(callSession.RingingDateUtc).local().format('DD.MM.YYYY HH:mm') : null;
        ci.ignored = ko.observable(callSession.Ignored);
        ci.ticketRelations = ko.observable(callSession.TicketRelations);
        ci.potentialUsers = callSession.PotentialUsers;
        ci.selectedUser = ko.observable(callSession.SelectedUser);

        ci.copyNoteToNewTicket = ko.observable(callSession.CopyNoteToNewTicket);
        ci.latestNotSavedNote = ko.observable(null);
        ci.note = ko.observable(callSession.Note);
        ci.noteLimited = ko.pureComputed(function () {
            return this.note();
        }, ci).extend({ rateLimit: 1000 });

        subscriptions.push(ci.note.subscribe(function (newNote) {
            ci.latestNotSavedNote(newNote);
        }));

        subscriptions.push(ci.noteLimited.subscribe(function (newNote) {
            $.connection.callHub.server.updateCallSessionNote(ci.uniqueId, newNote).done(function () {
                if (newNote === ci.latestNotSavedNote()) {
                    ci.latestNotSavedNote(null);
                }
            }).fail(function (error) {
                console.warn(error);
            });
        }));

        ci.callFields = ko.observable(callSession.CallFields ? callSession.CallFields.map(function (cfd) {
            return new CallFieldData(cfd, ci, subscriptions);
        }) : null);

        ci.categories = ko.observable(callSession.Categories ? callSession.Categories.map(function (cd) {
            return new CategoryData(cd, ci, subscriptions);
        }) : null);

        ci.setUserOfCallItem = function (userProviderKey) {
            return new Promise(function (resolve, reject) {
                $.connection.callHub.server.setUserOfCallSession(ci.uniqueId, userProviderKey).done(function () {
                    resolve();
                }).fail(function (error) {
                    console.warn(error);
                    reject(error);
                });
            });
        };

        ci.createTicketRelations = function (ticketIds) {
            return new Promise(function (resolve, reject) {
                for (var i = 0; i < ci.ticketRelations().length; i++) {
                    var ticketRelation = ci.ticketRelations()[i];
                    for (var j = 0; j < ticketIds.length; j++) {
                        var ticketId = ticketIds[j];
                        if (ticketRelation.TicketId === ticketId) {
                            ticketIds.splice(j--, 1);
                            break;
                        }
                    }
                }

                $.connection.callHub.server.createTicketRelationsOfCallSession(ci.uniqueId, ticketIds).done(function () {
                    resolve();
                }).fail(function (error) {
                    console.warn(error);
                    reject(error);
                });
            });
        };

        ci.deleteTicketRelation = function (ticketId) {
            return new Promise(function (resolve, reject) {
                $.connection.callHub.server.deleteTicketRelationOfCallSession(ci.uniqueId, ticketId).done(function () {
                    resolve();
                }).fail(function (error) {
                    console.warn(error);
                    reject(error);
                });
            });
        };

        ci.processCallItem = function () {
            var validationResult = ko.validation.group(ci, { deep: true });
            if (validationResult().length > 0) {
                validationResult.showAllMessages(true);
            }
            else {
                return new Promise(function (resolve, reject) {
                    $.connection.callHub.server.processCallSession(ci.uniqueId).done(function () {
                        resolve();
                    }).fail(function (error) {
                        console.warn(error);
                        reject(error);
                    });

                });
            }
        };

        ci.dispose = function () {
            subscriptions.forEach(function (subscription) {
                subscription.dispose();
            });
        };
    }

    function QueueRelation(name, isPaused, isLogged) {
        var qr = this;
        qr.name = name;
        qr.isPaused = ko.observable(isPaused);
        qr.isLogged = ko.observable(isLogged);
        qr.isPending = ko.observable(false);
        qr.isLoggedText = ko.pureComputed(function () {
            if (qr.isPending())
                return "NASTAVOVÁNÍ";

            if (qr.isLogged())
                return "PŘIHLÁŠENÝ";

            return "ODHLÁŠENÝ";

        }, this);


        function onNewIsLogged(newIsLogged) {
            qr.isPending(true);

            if (newIsLogged) {
                $.connection.callHub.server.logIn(qr.name).fail(function (error) {
                    console.warn(error);
                });
            }
            else {
                $.connection.callHub.server.logOut(qr.name).fail(function (error) {
                    console.warn(error);
                });
            }
        }

        qr.setNewIsLogged = function (newIsLogged) {
            loginSubscription.dispose();

            qr.isPending(false);
            qr.isLogged(newIsLogged);

            loginSubscription = qr.isLogged.subscribe(onNewIsLogged);
        };

        var loginSubscription = qr.isLogged.subscribe(onNewIsLogged);
    }

    function setSummary(summary) {
        //console.log(summary);
        setPausesAndQueueRelations(summary);

        var callItems = [];
        if (summary.CallUser.CallSessions)
        {
            for (var i = 0; i < summary.CallUser.CallSessions.length; i++) {
                let serverCallSession = summary.CallUser.CallSessions[i];
                callItems.push(new CallItem(serverCallSession));
            }
        }

        self.callItems(callItems);
        self.callItemsOverallSum(summary.CallUser.CallSessionsOverallSum);
    }

    function setPausesAndQueueRelations(summary) {
        self.pauses(summary.Pauses.concat([{ Reason: callSummary.NO_PAUSE_ID }]));
        setTimeout(function () {
            self.selectedPause(summary.CallUser.SumPauseReason ? summary.CallUser.SumPauseReason : callSummary.NO_PAUSE_ID);
        }, 0);

        var queueRelations = [];
        for (var i = 0; i < summary.CallUser.QueueRelations.length; i++) {
            let serverQueueRelation = summary.CallUser.QueueRelations[i];
            queueRelations.push(new QueueRelation(serverQueueRelation.Name, serverQueueRelation.IsPaused, serverQueueRelation.IsLogged));
        }

        self.queueRelations(queueRelations);
    }

    function getSummary() {
        return new Promise(function (resolve, reject) {
            self.isLoading(true);
            $.connection.callHub.server.getSummary(typeof callDashboard !== 'undefined').done(function (summary) {
                if (summary) {
                    setSummary(summary);
                }

                self.isLoading(false);
                selectDefaultCallItem();
                resolve();
            }).fail(function (error) {
                console.warn(error);
                self.isLoading(false);
                reject();
            });
        });
    }

    var initialized = false;
    self.init = function () {
        if (initialized)
            return;

        initialized = true;
        return new Promise(function (resolve, reject) {
            $.connection.callHub.client.logout = function (queue) {
                setQueueRelation(callSummary.callUserStateEnum.AGENT_LOGOUT, queue);
            };

            $.connection.callHub.client.login = function (queue) {
                setQueueRelation(callSummary.callUserStateEnum.AGENT_LOGIN, queue);
            };

            $.connection.callHub.client.pause = function (queue, pauseName) {
                setQueueRelation(callSummary.callUserStateEnum.AGENT_PAUSED, queue, pauseName);
            };

            $.connection.callHub.client.unpause = function (queue) {
                setQueueRelation(callSummary.callUserStateEnum.AGENT_UNPAUSED, queue);
            };

            $.connection.callHub.client.callSession = function (callSession) {
                setCallItem(callSession);
            };

            $.connection.callHub.client.setPausesAndQueueRelations = setPausesAndQueueRelations;
            $.connection.callHub.client.setSummary = setSummary;

            $.connection.callHub.client.ignoreCurrentCallSessions = function () {
                for (let i = 0; i < self.callItems().length; i++) {
                    let callItem = self.callItems()[i];
                    callItem.ignored(true);
                }
            };

            $.connection.callHub.client.callSessionProcessed = function (uniqueId) {
                var callItem = getCallItem(uniqueId);
                if (callItem) {
                    self.callItems.remove(callItem);
                    selectDefaultCallItem();
                }

                self.callItemsOverallSum(self.callItemsOverallSum() - 1);
            };

            $.connection.callHub.client.userOfCallSessionSet = function (uniqueId, user) {
                var callItem = getCallItem(uniqueId);
                if (callItem)
                    callItem.selectedUser(user);
            };

            $.connection.callHub.client.ticketRelationsOfCallSessionSet = function (uniqueId, ticketRelations) {
                var callItem = getCallItem(uniqueId);
                if (callItem)
                    callItem.ticketRelations(ticketRelations);
            };

            resolve({
                initCallback: getSummary
            });
        });
    };
}